/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright (c) 2019, MikroElektronika - www.mikroe.com
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * \file
 *
 */

#include "dac.h"

// ------------------------------------------------------------- PRIVATE MACROS 

#define DAC_DUMMY 0

// -------------------------------------------------------------- PRIVATE TYPES


// ------------------------------------------------------------------ CONSTANTS


// ------------------------------------------------------------------ VARIABLES


// ---------------------------------------------- PRIVATE FUNCTION DECLARATIONS 

static void dac_generic_transfer ( dac_t *ctx, 
                                   spi_master_transfer_data_t *block );

// ------------------------------------------------ PUBLIC FUNCTION DEFINITIONS

void dac_cfg_setup ( dac_cfg_t *cfg )
{
    // Communication gpio pins 

    cfg->sck = HAL_PIN_NC;
    cfg->miso = HAL_PIN_NC;
    cfg->mosi = HAL_PIN_NC;
    cfg->cs = HAL_PIN_NC;

    cfg->spi_speed = SPI_MEDIUM; 
    cfg->spi_mode = SPI_MODE_0;
}

DAC_RETVAL dac_init ( dac_t *ctx, dac_cfg_t *cfg )
{
    spi_master_config_t spi_cfg;

    spi_master_configure_default( &spi_cfg );
    spi_cfg.mode        = cfg->spi_mode;
    spi_cfg.speed       = cfg->spi_speed;
    spi_cfg.pins.sck    = cfg->sck;
    spi_cfg.pins.miso   = cfg->miso;
    spi_cfg.pins.mosi   = cfg->mosi;
    spi_cfg.chip_select = cfg->cs;

    if (  spi_master_open( &ctx->spi, &spi_cfg ) != SPI_SUCCESS )
    {
        return DAC_INIT_ERROR;
    }

    spi_master_set_dummy_data( &ctx->spi, DAC_DUMMY );

    return DAC_OK;
}

// Set voltage function 
void dac_set_voltage ( dac_t *ctx, uint16_t v_out )
{
    spi_master_transfer_data_t block;
    uint8_t tx_buf[ 2 ];
    
    tx_buf[ 0 ] = ( v_out >> 8 ) & DAC_4_BIT_DATA;
    tx_buf[ 0 ] |= DAC_APPLY_SETTINGS;
    tx_buf[ 1 ] = v_out & DAC_8_BIT_DATA;

    block.tx_length = 2;
    block.tx_buffer = tx_buf;
    block.rx_length = 0;

    dac_generic_transfer( ctx, &block );
}

// Function write procentage value 
void dac_set_voltage_pct ( dac_t *ctx, uint8_t v_out_pct )
{
    uint32_t value;
    uint16_t v_out;

    if ( v_out_pct > DAC_PERCENTAGE )
    {
        v_out_pct %= DAC_PERCENTAGE;
    }
    
    value = v_out_pct;
    
    value *= DAC_ONE_STEP_VAL;
    value /= DAC_CLAC_VAL;
    v_out = ( uint16_t )value;

    dac_set_voltage( ctx, v_out );
}

// ----------------------------------------------- PRIVATE FUNCTION DEFINITIONS

static void dac_generic_transfer ( dac_t *ctx, 
                                   spi_master_transfer_data_t *block )
{
    spi_master_start(&ctx->spi);
    spi_master_transfer(&ctx->spi, block);
    spi_master_stop(&ctx->spi);    
}

// ------------------------------------------------------------------------- END

